//+------------------------------------------------------------------+
//|                                                   00001 Mult.mq5 |
//+------------------------------------------------------------------+
//|                                                  MACD Sample.mq5 |
//|                   Copyright 2009-2017, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright   "Copyright 2009-2017, MetaQuotes Software Corp."
#property link        "http://www.mql5.com"
#property version     "5.50"
#property description "It is important to make sure that the expert works with a normal"
#property description "chart and the user did not make any mistakes setting input"
#property description "variables (Lots, TakeProfit, TrailingStop) in our case,"
#property description "we check TakeProfit on a chart of more than 2*trend_period bars"
//---
#define MACD_MAGIC  0
//---
#include <Trade\Trade.mqh>
#include <Trade\SymbolInfo.mqh>
#include <Trade\PositionInfo.mqh>
#include <Trade\AccountInfo.mqh>
//---
input double InpMaximumRisk   =0.02; // Maximum Risk in percentage
input int    InpMACDOpenLevel =3;    // MACD open level (in pips)
input int    InpMACDCloseLevel=2;    // MACD close level (in pips)
input int    InpMATrendPeriod =26;   // MA trend period
//---
int    m_handle_macd[]= {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
int    m_handle_ema[]= {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
double LotsRisk[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
string symb_name[]= {"EURUSD","GBPUSD","USDCHF","USDJPY","USDCAD","AUDUSD","AUDNZD",
                     "AUDCAD","AUDCHF","AUDJPY","CHFJPY","EURGBP","EURAUD","EURCHF",
                     "EURJPY","EURNZD","EURCAD","GBPCHF","GBPJPY","CADCHF","GBPAUD"
                    };
int ExtTimeOut=10; // time out in seconds between trade operations
//+------------------------------------------------------------------+
//| MACD Sample expert class                                         |
//+------------------------------------------------------------------+
class CSampleExpert
  {
protected:
   double            m_adjusted_point;             // point value adjusted for 3 or 5 points
   CTrade            m_trade;                      // trading object
   CSymbolInfo       m_symbol;                     // symbol info object
   CPositionInfo     m_position;                   // trade position object
   CAccountInfo      m_account;                    // account info wrapper
   //--- indicator buffers
   double            m_buff_MACD_main[];           // MACD indicator main buffer
   double            m_buff_MACD_signal[];         // MACD indicator signal buffer
   double            m_buff_EMA[];                 // EMA indicator buffer
   //--- indicator data for processing
   double            m_macd_current;
   double            m_macd_previous;
   double            m_signal_current;
   double            m_signal_previous;
   double            m_ema_current;
   double            m_ema_previous;
   //---
   double            m_macd_open_level;
   double            m_macd_close_level;

public:
                     CSampleExpert(void);
                    ~CSampleExpert(void);
   bool              Init(void);
   void              Deinit(void);
   bool              Processing(void);
   bool              Processing1(string symbol,int handle_macd,int handle_ema);

protected:
   bool              InitCheckParameters(const int digits_adjust);
   bool              InitIndicators(void);
   bool              LongOpened(string symbol);
   bool              ShortOpened(string symbol);
   //+------------------------------------------------------------------+
   //| Calculate optimal lot size                                       |
   //+------------------------------------------------------------------+
   double            TradeSizeOptimized(string symbol,double risk)
     {
      double price=0.0;
      double margin=0.0;
      //--- select lot size
      if(!SymbolInfoDouble(symbol,SYMBOL_ASK,price))
         return(0.0);
      if(!OrderCalcMargin(ORDER_TYPE_BUY,symbol,1.0,price,margin))
         return(0.0);
      if(margin<=0.0)
         return(0.0);
      double lot=NormalizeDouble(AccountInfoDouble(ACCOUNT_MARGIN_FREE)*risk/margin,2);
      //--- normalize and check limits
      double stepvol=SymbolInfoDouble(symbol,SYMBOL_VOLUME_STEP);
      lot=stepvol*NormalizeDouble(lot/stepvol,0);
      double minvol=SymbolInfoDouble(symbol,SYMBOL_VOLUME_MIN);
      if(lot<minvol)
         lot=minvol;
      double maxvol=SymbolInfoDouble(symbol,SYMBOL_VOLUME_MAX);
      if(lot>maxvol)
         lot=maxvol;
      //--- return trading volume
      return(lot);
     }
  };
//--- global expert
CSampleExpert ExtExpert;
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CSampleExpert::CSampleExpert(void) : m_adjusted_point(0),
   m_macd_current(0),
   m_macd_previous(0),
   m_signal_current(0),
   m_signal_previous(0),
   m_ema_current(0),
   m_ema_previous(0),
   m_macd_open_level(0),
   m_macd_close_level(0)
  {
   ArraySetAsSeries(m_buff_MACD_main,true);
   ArraySetAsSeries(m_buff_MACD_signal,true);
   ArraySetAsSeries(m_buff_EMA,true);
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CSampleExpert::~CSampleExpert(void)
  {
  }
//+------------------------------------------------------------------+
//| Initialization and checking for input parameters                 |
//+------------------------------------------------------------------+
bool CSampleExpert::Init(void)
  {
   for(int i=0; i<20; i++)
     {
      //--- initialize common information
      m_symbol.Name(symb_name[i]);                  // symbol
      m_trade.SetExpertMagicNumber(MACD_MAGIC); // magic
      m_trade.SetMarginMode();
      m_trade.SetTypeFillingBySymbol(symb_name[i]);
      //--- tuning for 3 or 5 digits
      int digits_adjust=1;
      if(m_symbol.Digits()==3 || m_symbol.Digits()==5)
         digits_adjust=10;
      m_adjusted_point=m_symbol.Point()*digits_adjust;
      //--- set default deviation for trading in adjusted points
      m_macd_open_level =InpMACDOpenLevel*m_adjusted_point;
      m_macd_close_level=InpMACDCloseLevel*m_adjusted_point;
      //--- set default deviation for trading in adjusted points
      m_trade.SetDeviationInPoints(3*digits_adjust);
      //---
      if(!InitCheckParameters(digits_adjust))
         return(false);
      if(!InitIndicators())
         return(false);
     }
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Checking for input parameters                                    |
//+------------------------------------------------------------------+
bool CSampleExpert::InitCheckParameters(const int digits_adjust)
  {
   for(int i=0; i<20; i++)
     {
      LotsRisk[i]=TradeSizeOptimized(symb_name[i],InpMaximumRisk);
      //--- check for right lots amount
      if(LotsRisk[i]<m_symbol.LotsMin() || LotsRisk[i]>m_symbol.LotsMax())
        {
         printf("Lots amount must be in the range from %f to %f",m_symbol.LotsMin(),m_symbol.LotsMax());
         return(false);
        }
      if(MathAbs(LotsRisk[i]/m_symbol.LotsStep()-MathRound(LotsRisk[i]/m_symbol.LotsStep()))>1.0E-10)
        {
         printf("Lots amount is not corresponding with lot step %f",m_symbol.LotsStep());
         return(false);
        }
     }
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Initialization of the indicators                                 |
//+------------------------------------------------------------------+
bool CSampleExpert::InitIndicators(void)
  {
   for(int i=0; i<20; i++)
     {
      //--- create MACD indicator
      m_handle_macd[i]=iMACD(symb_name[i],0,12,26,9,PRICE_CLOSE);
      if(m_handle_macd[i]==INVALID_HANDLE)
        {
         printf("Error creating MACD indicator");
         return(false);
        }
      //--- create EMA indicator and add it to collection
      m_handle_ema[i]=iMA(symb_name[i],0,InpMATrendPeriod,0,MODE_EMA,PRICE_CLOSE);
      if(m_handle_ema[i]==INVALID_HANDLE)
        {
         printf("Error creating EMA indicator");
         return(false);
        }
     }
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Check for long position opening                                  |
//+------------------------------------------------------------------+
bool CSampleExpert::LongOpened(string symbol)
  {
   bool res=false;
//--- check for long position (BUY) possibility
   if(m_macd_current<0)
      if(m_macd_current>m_signal_current && m_macd_previous<m_signal_previous)
         if(MathAbs(m_macd_current)>(m_macd_open_level) && m_ema_current>m_ema_previous)
           {
            double price=m_symbol.Ask();
            //--- check for free money
            if(m_account.FreeMarginCheck(symbol,ORDER_TYPE_BUY,TradeSizeOptimized(symbol,InpMaximumRisk),price)<0.0)
               printf("We have no money. Free Margin = %f",m_account.FreeMargin());
            else
              {
               //--- open position
               if(m_trade.PositionOpen(symbol,ORDER_TYPE_BUY,TradeSizeOptimized(symbol,InpMaximumRisk),price,0.0,0.0))
                  printf("Position by %s to be opened",symbol);
               else
                 {
                  printf("Error opening BUY position by %s : '%s'",symbol,m_trade.ResultComment());
                  printf("Open parameters : price=%f",price);
                 }
              }
            //--- in any case we must exit from expert
            res=true;
           }
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//| Check for short position opening                                 |
//+------------------------------------------------------------------+
bool CSampleExpert::ShortOpened(string symbol)
  {
   bool res=false;
//--- check for short position (SELL) possibility
   if(m_macd_current>0)
      if(m_macd_current<m_signal_current && m_macd_previous>m_signal_previous)
         if(m_macd_current>(m_macd_open_level) && m_ema_current<m_ema_previous)
           {
            double price=m_symbol.Bid();
            //--- check for free money
            if(m_account.FreeMarginCheck(symbol,ORDER_TYPE_SELL,TradeSizeOptimized(symbol,InpMaximumRisk),price)<0.0)
               printf("We have no money. Free Margin = %f",m_account.FreeMargin());
            else
              {
               //--- open position
               if(m_trade.PositionOpen(symbol,ORDER_TYPE_SELL,TradeSizeOptimized(symbol,InpMaximumRisk),price,0.0,0.0))
                  printf("Position by %s to be opened",symbol);
               else
                 {
                  printf("Error opening SELL position by %s : '%s'",symbol,m_trade.ResultComment());
                  printf("Open parameters : price=%f",price);
                 }
              }
            //--- in any case we must exit from expert
            res=true;
           }
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//| main function returns true if any position processed             |
//+------------------------------------------------------------------+
bool CSampleExpert::Processing(void)
  {
   for(int i=0; i<20; i++)
     {
      Processing1(symb_name[i],m_handle_macd[i],m_handle_ema[i]);
     }
//--- exit without position processing
   return(false);
  }
//+------------------------------------------------------------------+
//| main function returns true if any position processed             |
//+------------------------------------------------------------------+
bool CSampleExpert::Processing1(string symbol,int handle_macd,int handle_ema)
  {
   MqlRates rt[2];
//--- go trading only for first ticks of new bar
   if(CopyRates(symbol,_Period,0,2,rt)!=2)
     {
      Print("CopyRates of ",symbol," failed, no history");
      return(false);
     }
   if(rt[1].tick_volume>1)
      return(true);
//--- refresh rates
   if(!m_symbol.RefreshRates())
      return(false);
//--- refresh indicators
   if(BarsCalculated(handle_macd)<2 || BarsCalculated(handle_ema)<2)
      return(false);
   if(CopyBuffer(handle_macd,0,0,2,m_buff_MACD_main)  !=2 ||
      CopyBuffer(handle_macd,1,0,2,m_buff_MACD_signal)!=2 ||
      CopyBuffer(handle_ema,0,0,2,m_buff_EMA)         !=2)
      return(false);
//--- data are put into internal variables
   m_macd_current   =m_buff_MACD_main[0];
   m_macd_previous  =m_buff_MACD_main[1];
   m_signal_current =m_buff_MACD_signal[0];
   m_signal_previous=m_buff_MACD_signal[1];
   m_ema_current    =m_buff_EMA[0];
   m_ema_previous   =m_buff_EMA[1];
//--- it is important to enter the market correctly,
   if(LongOpened(symbol))
      return(true);
//--- check for short position (SELL) possibility
   if(ShortOpened(symbol))
      return(true);
//--- exit without position processing
   return(false);
  }
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit(void)
  {
//--- create all necessary objects
   if(!ExtExpert.Init())
      return(INIT_FAILED);
//--- secceed
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert new tick handling function                                |
//+------------------------------------------------------------------+
void OnTick(void)
  {
   ExtExpert.Processing();
  }
//+------------------------------------------------------------------+
